==> bpc8583-1.27.tar.gz: bpc8583-1.27/bpc8583/isoClient.py <==
#!/usr/bin/env python

import sys
import os
import getopt
import time
import xml.etree.ElementTree as ET
from collections import OrderedDict

from bpc8583.ISO8583 import ISO8583, MemDump
from bpc8583.spec import IsoSpec, IsoSpec1987BPC
from bpc8583.terminal import Terminal
from bpc8583.card import Card
from bpc8583.transaction import Transaction
from bpc8583.tools import trace_passed, trace_failed


class isoClient:
    def __init__(self, term, card, transactions=None):
        self.term = term
        self.card = card
        self.transactions = transactions


    def set_verbosity_level(self, verbosity_level):
        """
        """
        self.verbosity = verbosity_level


    def run(self):
        """
        """
        if self.transactions:
            self._run_non_interactive()
        else:
            self._run_interactive() 


    def _run_non_interactive(self):
        """
        """
        self.term.connect()
        for trxn in self.transactions:
            self.term.send(trxn.get_data(), show_trace=False)
            if self.verbosity:
                trxn.trace(header='Request')

            data = self.term.recv(show_trace=False)
            IsoMessage = ISO8583(data[2:], IsoSpec1987BPC())

            # Checking response code
            if trxn.is_response_expected(IsoMessage.FieldData(39)):
                trace_passed(trxn.get_description(), show_colored_description=self.verbosity)
                if self.verbosity:
                    IsoMessage.Print(header='Response')
            else:
                trace_failed(trxn.get_description(), IsoMessage.FieldData(39), show_colored_description=self.verbosity)
                if not self.verbosity:
                    trxn.trace(header='Request')
                IsoMessage.Print(header='Response')
    
        self.term.close()


    def _user_input(self, hint):
        """
        python-version-independent wrapper to raw_input()/input()
        """
        if sys.version[0] != '3':
            return raw_input(hint)
        else:
            return input(hint)


    def _show_available_transactions(self):
        """
        """
        print('e: echo')
        print('b: balance inquiry')
        print('p: manual purchase')


    def _run_interactive(self):
        """
        Run transactions interactively (by asking user which transaction to run)
        """
        self.term.connect()
        self._show_available_transactions()

        while True:
            trxn_type = self._user_input('\nEnter transaction to send: ')
        
            trxn = ''
            data = ''
            if trxn_type == 'e':
                trxn = Transaction('echo', self.card, self.term)
                trxn.trace()
        
            elif trxn_type == 'b':
                trxn = Transaction('balance', self.card, self.term)
                trxn.set_PIN(self._user_input('Enter PIN: '))
                trxn.trace()
        
            elif trxn_type == 'p':
                default_amount = 20000
                amount = self._user_input('Enter transaction amount ({} by default): '.format(default_amount))
                if not amount:
                    amount = default_amount

                trxn = Transaction('purchase', self.card, self.term)
                trxn.set_PIN(self._user_input('Enter PIN: '))
                trxn.set_amount(amount)
                trxn.trace()

            elif trxn_type == 'q':
                break

            else:
                print('Unknown transaction. Available transactions are:')
                self._show_available_transactions()
                continue
                
            self.term.send(trxn.get_data(), show_trace=verbosity)
            data = self.term.recv(show_trace=verbosity)
        
            IsoMessage = ISO8583(data[2:], IsoSpec1987BPC())
            IsoMessage.Print()
        
        self.term.close()


def parse_transaction_item(trxn, term, cards):
    """
    """
    t = None
    card = None
    card_description = None
    icc_trxn = None

    #card = cards[trxn.attrib['card']]
    try:
        card_description = trxn.attrib['card']
    except KeyError:
        for key, value in cards.items():
            card_description = value.get_description()
            break
    
    try:
        card = cards[card_description]
    except KeyError:
        if card_description:
            print('Card \'' + card_description + '\' not found.')
        for key, value in cards.items():
            card = value
            break    

    try:
        icc_trxn = trxn.attrib['icc']
    except:
        pass

    try:
        t = Transaction(trxn.attrib['type'], card, term, icc_trxn=icc_trxn)
    except KeyError:
        print('Error parsing {}: transaction type is not set'.format(filename))
        sys.exit()

    try:
        t.set_description(trxn.attrib['description'])
    except KeyError:
        pass

    for attrib in trxn:
        if attrib.tag.lower() == 'amount':
            t.set_amount(attrib.text)
        elif attrib.tag.lower() == 'pin':
            t.set_PIN(attrib.text)
        elif attrib.tag.lower() == 'currency':
            t.set_currency(attrib.text)
        elif attrib.tag.lower() == 'response_code':
            t.set_expected_code(attrib.text)
        elif attrib.tag.lower() == 'response_action':
            if not t.set_expected_action(attrib.text):
                print('Unknown response action: {}'.format(attrib.text))

    return t


def parse_card_data(card):
    """
    """
    pan = None
    expiry_date = None
    service_code = None
    PVVKi = None
    PVV = None
    CVV = None
    discr_data = None
    description = ''
    try:
        description = card.attrib['description']
    except:
        pass

    for attrib in card:
        if attrib.tag.lower() == 'pan':
            pan = attrib.text
        elif attrib.tag.lower() == 'expiry_date':
            expiry_date = attrib.text
        elif attrib.tag.lower() == 'service_code':
            service_code = attrib.text
        elif attrib.tag.lower() == 'pvvki':
            PVVKi = attrib.text
        elif attrib.tag.lower() == 'pvv':
            PVV = attrib.text
        elif attrib.tag.lower() == 'cvv':
            CVV = attrib.text
        elif attrib.tag.lower() == 'discr_data':
            discr_data = attrib.text

    return Card(pan=pan, expiry_date=expiry_date, service_code=service_code, pvvki=PVVKi, PVV=PVV, CVV=CVV, discretionary_data=discr_data, description=description)
            

def parse_data_file(filename, term):
    """
    """
    data_tree = ET.parse(filename)
    data_root = data_tree.getroot()

    cards = OrderedDict()
    for item in data_root:
        if item.tag == 'card':
            c = parse_card_data(item)
            if c:
                cards[c.get_str_card_number()] = c
                if c.get_description():
                    cards[c.get_description()] = c


    if not cards:
        # No <card> items in transaction file:
        c = Card()
        cards[c.get_str_card_number()] = c

    transactions = []
    for item in data_root:
        if item.tag == 'trxn':
            t = parse_transaction_item(item, term, cards)
            if t:
                transactions.append(t)

    return transactions


def show_help(name):
    """
    Show help and basic usage
    """
    print('Usage: python3 {} [OPTIONS]... '.format(name))
    print('ISO8583 message client')
    print('  -v, --verbose\t\tRun transactions verbosely')
    print('  -p, --port=[PORT]\t\tTCP port to connect to, 1337 by default')
    print('  -s, --server=[IP]\t\tIP of the ISO host to connect to, 127.0.0.1 by default')
    print('  -t, --terminal=[ID]\t\tTerminal ID (used in DE 41 ISO field, 10001337 by default)')
    print('  -m, --merchant=[ID]\t\tMerchant ID (used in DE 42 ISO field, 999999999999001 by default)')
    print('  -k, --key=[KEY]\t\tTerminal key (\'DEADBEEF DEADBEEF DEADBEEF DEADBEEF\' by default)')
    print('  -f, --file=[file.xml]\t\tUse transaction data from the given XML-file')


if __name__ == '__main__':
    verbosity = False
    ip = None
    port = None
    terminal_id = None
    merchant_id = None
    terminal_key = None
    data_file = None
    transactions = None

    try:
        optlist, args = getopt.getopt(sys.argv[1:], 'hp:s:t:m:k:f:v', ['help', 'port=', 'server=', 'terminal=', 'merchant=', 'key=', 'file=', 'verbose'])
        for opt, arg in optlist:
            if opt in ('-v', '--verbose'):
                verbosity = True

            if opt in ('-h', '--help'):
                show_help(sys.argv[0])
                sys.exit()
    
            elif opt in ('-p', '--port'):
                port = arg
    
            elif opt in ('-s', '--server'):
                ip = arg
    
            elif opt in ('-t', '--terminal'):
                terminal_id = arg
    
            elif opt in ('-m', '--merchant'):
                merchant_id = arg

            elif opt in ('-k', '--key'):
                terminal_key = arg

            elif opt in ('-f', '--file'):
                data_file = arg

    except getopt.GetoptError:
        show_help(sys.argv[0])
        sys.exit()
    
    term = Terminal(host=ip, port=port, id=terminal_id, merchant=merchant_id, key=terminal_key)
    if data_file:
        transactions = parse_data_file(data_file, term)

    pos = isoClient(term, Card(), transactions)
    pos.set_verbosity_level(verbosity)
    pos.run()

==> bpc8583-1.33.tar.gz: bpc8583-1.33/bpc8583/isoClient.py <==
#!/usr/bin/env python

import sys
import os
import getopt
import time
import xml.etree.ElementTree as ET
from collections import OrderedDict

from bpc8583.ISO8583 import ISO8583, MemDump
from bpc8583.spec import IsoSpec, IsoSpec1987BPC
from bpc8583.terminal import Terminal
from bpc8583.card import Card
from bpc8583.transaction import Transaction
from bpc8583.tools import trace_passed, trace_failed


class isoClient:
    def __init__(self, term, card, transactions=None):
        self.term = term
        self.card = card
        self.transactions = transactions


    def set_verbosity_level(self, verbosity_level):
        """
        """
        self.verbosity = verbosity_level


    def run(self):
        """
        """
        if self.transactions:
            self._run_non_interactive()
        else:
            self._run_interactive() 


    def _run_non_interactive(self):
        """
        """
        self.term.connect()
        for trxn in self.transactions:
            # Re-encrypt PIN block if terminal key has changed
            PIN = trxn.get_PIN()
            if PIN:
                trxn.set_PIN(PIN)

            self.term.send(trxn.get_data(), show_trace=False)
            if self.verbosity:
                trxn.trace(header='Request')

            data = self.term.recv(show_trace=False)
            IsoMessage = ISO8583(data[2:], IsoSpec1987BPC())

            # Checking response code
            if trxn.is_response_expected(IsoMessage.FieldData(39)):
                trace_passed(trxn.get_description(), show_colored_description=self.verbosity)
                if self.verbosity:
                    IsoMessage.Print(header='Response')
            else:
                trace_failed(trxn.get_description(), IsoMessage.FieldData(39), show_colored_description=self.verbosity)
                if not self.verbosity:
                    trxn.trace(header='Request')
                IsoMessage.Print(header='Response')

            # Network management processing
            if IsoMessage.MTI() == '0810':
                if IsoMessage.FieldData(53):
                    new_terminal_key = IsoMessage.FieldData(53) # e.g. '31361828DF416311C184F97A2E5E3B6DD62C', first two bytes is length
                    self.term.set_terminal_key(new_terminal_key[4:])
    
        self.term.close()


    def _user_input(self, hint):
        """
        python-version-independent wrapper to raw_input()/input()
        """
        if sys.version[0] != '3':
            return raw_input(hint)
        else:
            return input(hint)


    def _show_available_transactions(self):
        """
        """
        print('e: echo')
        print('b: balance inquiry')
        print('p: manual purchase')


    def _run_interactive(self):
        """
        Run transactions interactively (by asking user which transaction to run)
        """
        self.term.connect()
        self._show_available_transactions()

        while True:
            trxn_type = self._user_input('\nEnter transaction to send: ')
        
            trxn = ''
            data = ''
            if trxn_type == 'e':
                trxn = Transaction('echo', self.card, self.term)
                trxn.trace()
        
            elif trxn_type == 'b':
                trxn = Transaction('balance', self.card, self.term)
                trxn.set_PIN(self._user_input('Enter PIN: '))
                trxn.trace()
        
            elif trxn_type == 'p':
                default_amount = 20000
                amount = self._user_input('Enter transaction amount ({} by default): '.format(default_amount))
                if not amount:
                    amount = default_amount

                trxn = Transaction('purchase', self.card, self.term)
                trxn.set_PIN(self._user_input('Enter PIN: '))
                trxn.set_amount(amount)
                trxn.trace()

            elif trxn_type == 'q':
                break

            else:
                print('Unknown transaction. Available transactions are:')
                self._show_available_transactions()
                continue
                
            self.term.send(trxn.get_data(), show_trace=verbosity)
            data = self.term.recv(show_trace=verbosity)
        
            IsoMessage = ISO8583(data[2:], IsoSpec1987BPC())
            IsoMessage.Print()
        
        self.term.close()


def parse_transaction_item(trxn, term, cards):
    """
    """
    t = None
    card = None
    card_description = None
    icc_trxn = None

    #card = cards[trxn.attrib['card']]
    try:
        card_description = trxn.attrib['card']
    except KeyError:
        for key, value in cards.items():
            card_description = value.get_description()
            break
    
    try:
        card = cards[card_description]
    except KeyError:
        if card_description:
            print('Card \'' + card_description + '\' not found.')
        for key, value in cards.items():
            card = value
            break    

    try:
        icc_trxn = trxn.attrib['icc']
    except:
        pass

    try:
        t = Transaction(trxn.attrib['type'], card, term, icc_trxn=icc_trxn)
    except KeyError:
        print('Error parsing {}: transaction type is not set'.format(filename))
        sys.exit()

    try:
        t.set_description(trxn.attrib['description'])
    except KeyError:
        pass

    for attrib in trxn:
        if attrib.tag.lower() == 'amount':
            t.set_amount(attrib.text)
        elif attrib.tag.lower() == 'pin':
            t.set_PIN(attrib.text)
        elif attrib.tag.lower() == 'currency':
            t.set_currency(attrib.text)
        elif attrib.tag.lower() == 'response_code':
            t.set_expected_code(attrib.text)
        elif attrib.tag.lower() == 'response_action':
            if not t.set_expected_action(attrib.text):
                print('Unknown response action: {}'.format(attrib.text))

    return t


def parse_card_data(card):
    """
    """
    pan = None
    expiry_date = None
    service_code = None
    PVVKi = None
    PVV = None
    CVV = None
    discr_data = None
    description = ''
    try:
        description = card.attrib['description']
    except:
        pass

    for attrib in card:
        if attrib.tag.lower() == 'pan':
            pan = attrib.text
        elif attrib.tag.lower() == 'expiry_date':
            expiry_date = attrib.text
        elif attrib.tag.lower() == 'service_code':
            service_code = attrib.text
        elif attrib.tag.lower() == 'pvvki':
            PVVKi = attrib.text
        elif attrib.tag.lower() == 'pvv':
            PVV = attrib.text
        elif attrib.tag.lower() == 'cvv':
            CVV = attrib.text
        elif attrib.tag.lower() == 'discr_data':
            discr_data = attrib.text

    return Card(pan=pan, expiry_date=expiry_date, service_code=service_code, pvvki=PVVKi, PVV=PVV, CVV=CVV, discretionary_data=discr_data, description=description)
            

def parse_data_file(filename, term):
    """
    """
    data_tree = ET.parse(filename)
    data_root = data_tree.getroot()

    cards = OrderedDict()
    for item in data_root:
        if item.tag == 'card':
            c = parse_card_data(item)
            if c:
                cards[c.get_str_card_number()] = c
                if c.get_description():
                    cards[c.get_description()] = c


    if not cards:
        # No <card> items in transaction file:
        c = Card()
        cards[c.get_str_card_number()] = c

    transactions = []
    for item in data_root:
        if item.tag == 'trxn':
            t = parse_transaction_item(item, term, cards)
            if t:
                transactions.append(t)

    return transactions


def show_help(name):
    """
    Show help and basic usage
    """
    print('Usage: python3 {} [OPTIONS]... '.format(name))
    print('ISO8583 message client')
    print('  -v, --verbose\t\tRun transactions verbosely')
    print('  -p, --port=[PORT]\t\tTCP port to connect to, 1337 by default')
    print('  -s, --server=[IP]\t\tIP of the ISO host to connect to, 127.0.0.1 by default')
    print('  -t, --terminal=[ID]\t\tTerminal ID (used in DE 41 ISO field, 10001337 by default)')
    print('  -m, --merchant=[ID]\t\tMerchant ID (used in DE 42 ISO field, 999999999999001 by default)')
    print('  -k, --terminal-key=[KEY]\t\tTerminal key (\'DEADBEEF DEADBEEF DEADBEEF DEADBEEF\' by default)')
    print('  -K, --master-key=[KEY]\t\Master key (\'ABABABAB CDCDCDCD EFEFEFEF AEAEAEAE\' by default)')
    print('  -f, --file=[file.xml]\t\tUse transaction data from the given XML-file')


if __name__ == '__main__':
    verbosity = False
    ip = None
    port = None
    terminal_id = None
    merchant_id = None
    terminal_key = None
    master_key = None
    data_file = None
    transactions = None

    try:
        optlist, args = getopt.getopt(sys.argv[1:], 'hp:s:t:m:k:K:f:v', ['help', 'port=', 'server=', 'terminal=', 'merchant=', 'terminal-key=', 'master-key=', 'file=', 'verbose'])
        for opt, arg in optlist:
            if opt in ('-v', '--verbose'):
                verbosity = True

            if opt in ('-h', '--help'):
                show_help(sys.argv[0])
                sys.exit()
    
            elif opt in ('-p', '--port'):
                port = arg
    
            elif opt in ('-s', '--server'):
                ip = arg
    
            elif opt in ('-t', '--terminal'):
                terminal_id = arg
    
            elif opt in ('-m', '--merchant'):
                merchant_id = arg

            elif opt in ('-k', '--terminal-key'):
                terminal_key = arg

            elif opt in ('-K', '--master-key'):
                master_key = arg

            elif opt in ('-f', '--file'):
                data_file = arg

    except getopt.GetoptError:
        show_help(sys.argv[0])
        sys.exit()
    
    term = Terminal(host=ip, port=port, id=terminal_id, merchant=merchant_id, terminal_key=terminal_key, master_key=master_key, show_keys=True)
    if data_file:
        transactions = parse_data_file(data_file, term)

    pos = isoClient(term, Card(), transactions)
    pos.set_verbosity_level(verbosity)
    pos.run()
